@@ -50,9 +50,11 @@ import filecmp
 ## to inherit and do more complex things
 class BaseLoadCase(unittest.TestCase):
     def setUp(self):
+        self.tempdir = tempfile.mkdtemp()
         self.importDirSetUp()
         self.loadDirsSetUp()
-        self.exportparent = tempfile.mkdtemp()
+        self.exportparent = os.path.join(self.tempdir, 'exportparent')
+        os.mkdir(self.exportparent)
         self.repo = self.createRepository()
         self.url = 'file://' + self.repo
         ## idir is the relative path in repo where things get imported
@@ -60,30 +62,31 @@ class BaseLoadCase(unittest.TestCase):
         self.extra_args = []
 
     def importDirSetUp(self):
-        self.importdir = tempfile.mkdtemp()
+        self.importdir = os.path.join(self.tempdir, 'importdir')
+        os.mkdir(self.importdir)
 
     def loadDirsSetUp(self):
-        self.loaddirs = [tempfile.mkdtemp()]
+        self.loaddirs = [os.path.join(self.tempdir, 'loaddir0')]
+        os.mkdir(self.loaddirs[0])
         
     def tearDown(self):
-        for d in self.loaddirs + [self.importdir, self.repo, self.exportparent]:
-            if d:
-                shutil.rmtree(d)
+        shutil.rmtree(self.tempdir)
 
     def createRepository(self):
         svnadmin = '/usr/bin/svnadmin'
-        repo = tempfile.mkdtemp()
+        repo = os.path.join(self.tempdir, 'repo')
+        os.mkdir(repo)
         os.spawnl(os.P_WAIT, svnadmin, svnadmin, 'create', repo)
         return repo
 
     def exportRepository(self):
         svn = '/usr/bin/svn'
-        ret = os.spawnl(os.P_WAIT, svn, svn, 'export', self.url,
+        ret = os.spawnl(os.P_WAIT, svn, svn, 'export', '--quiet', self.url,
                         os.path.join(self.exportparent, 'export'))
         if ret == 0:
             return True
         else:
-            return None
+            return False
 
     def loaddir(self, ldir):
         svnloaddirs = "./svn-load"
@@ -113,6 +116,10 @@ class BaseLoadCase(unittest.TestCase):
         return haha
 
     def compare_dirs(self, a, b):
+        if os.path.islink(a) and not os.path.islink(b) or \
+           not os.path.islink(a) and os.path.islink(b):
+            return False
+
         c = filecmp.dircmp(a, b)
         funny_ok = self.funny_check(a, b, c.common_funny)
         if c.left_only or c.right_only or c.diff_files or not funny_ok:
@@ -134,7 +141,7 @@ class BaseLoadCase(unittest.TestCase):
         client = pysvn.Client()
         client.import_(self.importdir, self.url, 'Initial import')
         for d in self.loaddirs:
-            self.loaddir(d)
+            self.failUnless(self.loaddir(d))
             self.failUnless(self.exportRepository())
             self.failUnless(self.compare_trees(d, os.path.join(self.exportparent, 'export')))
 
@@ -237,7 +244,7 @@ class BaseMoveCase(BaseLoadCase):
         client = pysvn.Client()
         client.import_(self.importdir, self.url, 'Initial import')
         
-        self.loaddir(self.loaddirs[0])
+        self.failUnless(self.loaddir(self.loaddirs[0]))
         log = client.log(os.path.join(self.url, self.idir), limit=1,
                          discover_changed_paths=True)[0]
         changed_paths = log['changed_paths']
@@ -305,16 +312,22 @@ class MultipleStringMoves(BaseMoveCase):
         f.write("^bar$  'bang'\n")
         f.write("^baz$  'eek'\n")
 
-class SingleFileToBrokenSymLink(BaseLoadCase):
-    def importDirSetUp(self):
-        BaseLoadCase.importDirSetUp(self)
-        f = open(os.path.join(self.importdir, 'foo'), 'w')
-        f.write("bar\n")
-        f.close()
+class MoveMapWithDollarSign(BaseMoveCase):
+    files_old = [
+        'foo',
+        'bar$baz',
+    ]
+
+    files_new = [
+        'foo',
+        'baz',
+    ]
+
+    def makeMoveMap(self):
+        f = open(self.movemap, 'w')
+        f.write(r"^bar\$baz$  'baz'")
+        f.write("\n")
 
-    def loadDirsSetUp(self):
-        BaseLoadCase.loadDirsSetUp(self)
-        os.symlink('broken', os.path.join(self.loaddirs[0], 'foo'))
 
 class SingleFileToBrokenSymLink(BaseLoadCase):
     def importDirSetUp(self):
@@ -327,6 +340,31 @@ class SingleFileToBrokenSymLink(BaseLoadCase):
         BaseLoadCase.loadDirsSetUp(self)
         os.symlink('broken', os.path.join(self.loaddirs[0], 'foo'))
 
+class SingleFileToSymLink(BaseLoadCase):
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        f = open(os.path.join(self.importdir, 'foo'), 'w')
+        f.close()
+        f = open(os.path.join(self.importdir, 'bar'), 'w')
+        f.close()
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        f = open(os.path.join(self.loaddirs[0], 'foo'), 'w')
+        f.close()
+        os.symlink('foo', os.path.join(self.loaddirs[0], 'bar'))
+
+class SingleFileToDirectorySymLink(BaseLoadCase):
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        os.mkdir(os.path.join(self.importdir, 'foo'))
+        open(os.path.join(self.importdir, 'bar'), 'w').close()
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        os.mkdir(os.path.join(self.loaddirs[0], 'foo'))
+        os.symlink('foo', os.path.join(self.loaddirs[0], 'bar'))
+
 class BrokenSymLinkToFile(BaseLoadCase):
     def importDirSetUp(self):
         BaseLoadCase.importDirSetUp(self)
@@ -338,6 +376,117 @@ class BrokenSymLinkToFile(BaseLoadCase):
         f.write("bar\n")
         f.close()
 
+class SymLinkToFile(BaseLoadCase):
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        open(os.path.join(self.importdir, 'foo'), 'w').close()
+        os.symlink('foo', os.path.join(self.importdir, 'bar'))
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        open(os.path.join(self.loaddirs[0], 'foo'), 'w').close()
+        open(os.path.join(self.loaddirs[0], 'bar'), 'w').close()
+
+class DirectorySymLinkToFile(BaseLoadCase):
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        os.mkdir(os.path.join(self.importdir, 'foo'))
+        os.symlink('foo', os.path.join(self.importdir, 'bar'))
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        os.mkdir(os.path.join(self.loaddirs[0], 'foo'))
+        open(os.path.join(self.loaddirs[0], 'bar'), 'w').close()
+
+
+class ReimportSymlinkBase(BaseLoadCase):
+    def _fillDirectory(self, directory):
+        # overwrite in derived class
+        pass
+
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        self._fillDirectory(self.importdir)
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        self._fillDirectory(self.loaddirs[0])
+        # create files one alphabetically before and one after 'bar'
+        open(os.path.join(self.loaddirs[0], 'a-file'), 'w').close()
+        open(os.path.join(self.loaddirs[0], 'zoo-file'), 'w').close()
+        # create a subdirectory hierarchy
+        os.mkdir(os.path.join(self.loaddirs[0], 'zoo'))
+        os.mkdir(os.path.join(self.loaddirs[0], 'zoo', 'subzoo'))
+        os.mkdir(os.path.join(self.loaddirs[0], 'zoo', 'subzoo', 'subsubzoo'))
+        open(os.path.join(self.loaddirs[0], 'zoo', 'subzoo', 'subsubzoo',
+                          'subsubzoo-file'),
+             'w').close()
+
+class ReimportSymlink(ReimportSymlinkBase):
+    def _fillDirectory(self, directory):
+        open(os.path.join(directory, 'foo'), 'w').close()
+        os.symlink('foo', os.path.join(directory, 'bar'))
+
+class ReimportSymlinkToDirectory(ReimportSymlinkBase):
+    """svn-load version 0.9 fails on this and _commits_ an
+    _inconsistent_ content
+    """
+    def _fillDirectory(self, directory):
+        os.mkdir(os.path.join(directory, 'foo'))
+        os.symlink('foo', os.path.join(directory, 'bar'))        
+
+class ReimportBrokenSymlink(ReimportSymlinkBase):
+    def _fillDirectory(self, directory):
+        os.symlink('foo', os.path.join(directory, 'bar'))
+
+class SyblingDirSymlink(BaseLoadCase):
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        os.mkdir(os.path.join(self.loaddirs[0], 'foo'))
+        os.symlink('foo', os.path.join(self.loaddirs[0], 'bar'))
+
+class GlobalIgnoreFile(BaseLoadCase):
+    ## As of this writing, svn-load assumes that all files in a directory
+    ## will be added when the subdirectory is added, so it doesn't bother
+    ## descending down a tree it just added. However, some files are always
+    ## ignored by svn, even if no svn:ignore property exists. This is
+    ## controlled by the global-ignores parameter. svn-load blows up
+    ## on these files during its check_permissions pass, as it assumes it
+    ## can query their svn:executable property
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        os.mkdir(os.path.join(self.loaddirs[0], 'foo'))
+        f = open(os.path.join(self.loaddirs[0], 'foo', 'bar.o'), 'w')
+        f.write("baz\n")
+        f.close()
+
+## This test creates two load dirs: foo and foo2, then attempts to load
+## 'foo*'. On UNIX this should work because the shell expands the glob.
+## On Windows it should work because svn-load expands the glob
+class GlobHandling(BaseLoadCase):
+    def loadDirsSetUp(self):
+        self.loaddirparent = os.path.join(self.tempdir, 'loaddirparent')
+        os.mkdir(self.loaddirparent)
+        self.loaddirs = [os.path.join(self.loaddirparent, 'foo'),
+                         os.path.join(self.loaddirparent, 'foo2')]
+        for d in self.loaddirs:
+            os.mkdir(d)
+            f = open(os.path.join(d, 'test'), 'w')
+            f.write(d + '\n')
+            f.close()
+
+    def testLoad(self):
+        svnload = './svn-load'
+        client = pysvn.Client()
+        client.import_(self.importdir, self.url, 'Initial import')
+        os.system('%s %s / %s*' %
+                  (svnload, self.url, os.path.join(self.loaddirparent, "foo*")))
+        self.failUnless(self.exportRepository())
+        self.failUnless(self.compare_trees(self.loaddirs[-1], os.path.join(self.exportparent, 'export')))
 
 if __name__ == '__main__':
-    unittest.main()
+    try:
+        defaultTest = sys.argv[1]
+    except:
+        defaultTest = None
+    unittest.main(defaultTest=defaultTest)
